@@ -1,8 +1,6 @@
 import functools
-import inspect
 import threading
-import time
-from typing import Optional, Callable
+from typing import Optional, Callable, Union
 
 __all__ = [
 	'new_thread',
@@ -25,6 +23,7 @@ class FunctionThread(threading.Thread):
 				self.__return_value = target(*args_, **kwargs_)
 			except Exception as e:
 				self.__error = e
+				raise e from None
 
 		self._target = wrapped_target
 
@@ -38,17 +37,19 @@ class FunctionThread(threading.Thread):
 		return self.__return_value
 
 
-def new_thread(thread_name: Optional[str or Callable] = None):
+def new_thread(arg: Optional[Union[str, Callable]] = None):
 	"""
-	Use a new thread to execute the decorated function
-	The function return value will be set to the thread instance that executes this function
+	Use a new thread to execute the decorated function asynchronously
 	The name of the thread can be specified in parameter
+
+	The return value of the decorated function is changed to the thread instance that executes this function
+	The decorated function has 1 extra field:
+	- `original` field: stores the original undecorated function
 	"""
 	def wrapper(func):
 		@functools.wraps(func)  # to preserve the origin function information
 		def wrap(*args, **kwargs):
 			thread = FunctionThread(target=func, args=args, kwargs=kwargs, name=thread_name)
-			thread.setDaemon(True)
 			thread.start()
 			return thread
 		# bring the signature of the func to the wrap function
@@ -56,36 +57,12 @@ def new_thread(thread_name: Optional[str or Callable] = None):
 		misc_util.copy_signature(wrap, func)
 		wrap.original = func  # access this field to get the original function
 		return wrap
-	# Directly use @on_new_thread without ending brackets case
-	if isinstance(thread_name, Callable):
-		this_is_a_function = thread_name
+	# Directly use @new_thread without ending brackets case, e.g. @new_thread
+	if isinstance(arg, Callable):
 		thread_name = None
-		return wrapper(this_is_a_function)
-	# Use @on_new_thread with ending brackets case
-	return wrapper
+		return wrapper(arg)
+	# Use @new_thread with ending brackets case, e.g. @new_thread('A'), @new_thread()
+	else:
+		thread_name = arg
+		return wrapper
 
-
-def test():
-	@new_thread()
-	def bla1(value):
-		time.sleep(0.5)
-		print(threading.current_thread().getName() + ' ' + str(value))
-
-	@new_thread
-	def bla2(value):
-		time.sleep(0.5)
-		print(threading.current_thread().getName() + ' ' + str(value))
-
-	@new_thread('awa')
-	def bla3(value):
-		time.sleep(0.5)
-		print(threading.current_thread().getName() + ' ' + str(value))
-
-	print(bla1, bla1('1'), inspect.getfullargspec(bla1))
-	print(bla2, bla2('2'), inspect.getfullargspec(bla2))
-	print(bla3, bla3('3'), inspect.getfullargspec(bla3))
-	time.sleep(1)
-
-
-if __name__ == '__main__':
-	test()
