@@ -1,11 +1,13 @@
+from typing import Tuple, Optional, TypeVar, Callable
+
 DIVIDER = ' '
 
 
-def remove_divider_prefix(text):
+def remove_divider_prefix(text: str) -> str:
 	return text.lstrip(DIVIDER)
 
 
-def get_element(text):
+def get_element(text: str) -> str:
 	"""
 	"my test command" -> "my"
 	:type text: str
@@ -18,24 +20,25 @@ def get_element(text):
 		return text[:pos]
 
 
-def __get_var(text, func):
+T = TypeVar('T')
+
+
+def __get_var(text: str, func: Callable[[str], T]) -> Tuple[Optional[T], int]:
 	"""
 	Return value, char_read
 	value will be None if ValueError occurs
 	"""
-	origin_text = text
-	text = remove_divider_prefix(text)
 	arg = get_element(text)
 	try:
 		value = func(arg)
 	except ValueError:
 		value = None
-	return value, len(origin_text) - len(text) + len(arg)
+	return value, len(arg)
 
 
-def get_int(text):
+def get_int(text: str) -> Tuple[Optional[int], int]:
 	return __get_var(text, int)
 
 
-def get_float(text):
+def get_float(text: str) -> Tuple[Optional[float], int]:
 	return __get_var(text, float)
