@@ -1,5 +1,8 @@
 from abc import ABC
-from typing import Optional, Callable
+from typing import Optional, Union
+
+from mcdreforged.minecraft.rtext import RTextBase, RText, RColor
+from mcdreforged.utils.types import MessageText
 
 
 class CommandErrorBase(Exception, ABC):
@@ -17,23 +20,41 @@ class CommandError(CommandErrorBase, ABC):
 	"""
 	The basic error, for errors raising when a command source is executing a command
 	"""
-	def __init__(self, message: str, parsed_command: str, failed_command: str):
-		self.__message = message
-		self._parsed_command = parsed_command
-		self._failed_command = failed_command
-		self.__handled = False
+	def __init__(self, message: MessageText, parsed_command: str, failed_command: str):
+		#  !!something wroooong command
+		#  [--parsed--|-error-]
+		#  [------failed------]
+
+		self.__message: MessageText = message
+		self._parsed_command: str = parsed_command
+		self._failed_command: str = failed_command
+		self.__handled: bool = False
 
 	def __str__(self):
 		return '{}: {}<--'.format(self.__message, self._failed_command)
 
-	def to_mc_color_text(self):
-		return '§c{}: §r{}§c{}§4<--'.format(self.__message, self._parsed_command, self._failed_command[len(self._parsed_command):])
+	@property
+	def _error_command(self) -> str:
+		return self._failed_command[len(self._parsed_command):]
 
-	def get_translation_args(self) -> tuple:
+	def to_rtext(self) -> RTextBase:
+		return RTextBase.format(
+			'{}: {}{}{}',
+			RTextBase.from_any(self.__message).copy().set_color(RColor.red),
+			RText(self._parsed_command, RColor.dark_red),
+			RText(self._error_command, RColor.red),
+			RText('<--', RColor.dark_red)
+		)
+
+	def get_error_data(self) -> tuple:
+		"""
+		Data that might be helpful to the error display
+		Can be used in formatting processing
+		"""
 		return ()
 
-	def set_translated_message(self, key: str, translator: Callable[[str, tuple], str]):
-		self.__message = translator(key, self.get_translation_args())
+	def set_message(self, message: str):
+		self.__message = message
 
 	def get_parsed_command(self) -> str:
 		return self._parsed_command
@@ -79,12 +100,20 @@ class RequirementNotMet(CommandError):
 	"""
 	The specified requirement for the command source to enter this node is not met
 	"""
-	def __init__(self, parsed_command: str, failed_command: str, reason: Optional[str]):
-		self.__reason = reason if reason is not None else 'Requirement is not met'
+	__NO_REASON = RText('Requirement not met')
+
+	def __init__(self, parsed_command: str, failed_command: str, reason: Optional[MessageText]):
+		self.__reason: MessageText = reason if reason is not None else self.__NO_REASON
 		super().__init__(self.__reason, parsed_command, failed_command)
 
-	def get_translation_args(self) -> tuple:
-		return (self.__reason,)
+	def has_custom_reason(self) -> bool:
+		return self.__reason is not self.__NO_REASON
+
+	def get_reason(self) -> MessageText:
+		return self.__reason
+
+	def get_error_data(self) -> tuple:
+		return (self.get_reason(),)
 
 # -----------------
 #   Syntax things
@@ -96,10 +125,10 @@ class CommandSyntaxError(CommandError, ABC):
 	General illegal argument error
 	Used in integer parsing failure etc.
 	"""
-	def __init__(self, message: str, char_read: int):
-		super().__init__(message, '?', '?')
+	def __init__(self, message: str, char_read: Union[int, str]):
+		super().__init__(message, '', '?' if isinstance(char_read, int) else char_read)
 		self.message = message
-		self.char_read = char_read
+		self.char_read = char_read if isinstance(char_read, int) else len(char_read)
 
 	def set_parsed_command(self, parsed_command):
 		self._parsed_command = parsed_command
@@ -124,7 +153,7 @@ class LiteralNotMatch(CommandSyntaxError):
 
 
 class AbstractOutOfRange(IllegalArgument, ABC):
-	def __init__(self, message: str, char_read: int, value, range_l, range_r):
+	def __init__(self, message: str, char_read: Union[int, str], value, range_l, range_r):
 		"""
 		:param value: The actual value
 		:param range_l: The left boundary
@@ -135,8 +164,12 @@ class AbstractOutOfRange(IllegalArgument, ABC):
 		self.__range_l = range_l
 		self.__range_r = range_r
 
-	def get_translation_args(self) -> tuple:
-		return self.__value, self.__range_l if self.__range_l is not None else '', self.__range_r if self.__range_r is not None else ''
+	@classmethod
+	def _get_boundary_text(cls, value) -> str:
+		return str(value) if value is not None else '/'
+
+	def get_error_data(self) -> tuple:
+		return self.__value, self._get_boundary_text(self.__range_l), self._get_boundary_text(self.__range_r)
 
 
 # Number things
@@ -146,22 +179,22 @@ class NumberOutOfRange(AbstractOutOfRange):
 	"""
 	The parsed number value is out of the restriction range
 	"""
-	def __init__(self, char_read: int, value, range_l, range_r):
-		super().__init__('Value out of range [{}, {}]'.format(range_l, range_r), char_read, value, range_l, range_r)
+	def __init__(self, char_read: Union[int, str], value, range_l, range_r):
+		super().__init__('Value out of range [{}, {}]'.format(self._get_boundary_text(range_l), self._get_boundary_text(range_r)), char_read, value, range_l, range_r)
 
 
 class InvalidNumber(IllegalArgument):
-	def __init__(self, char_read: int):
+	def __init__(self, char_read: Union[int, str]):
 		super().__init__('Invalid number', char_read)
 
 
 class InvalidInteger(IllegalArgument):
-	def __init__(self, char_read: int):
+	def __init__(self, char_read: Union[int, str]):
 		super().__init__('Invalid integer', char_read)
 
 
 class InvalidFloat(IllegalArgument):
-	def __init__(self, char_read: int):
+	def __init__(self, char_read: Union[int, str]):
 		super().__init__('Invalid float', char_read)
 
 
@@ -172,15 +205,15 @@ class TextLengthOutOfRange(AbstractOutOfRange):
 	"""
 	The length of the given text is out of the restriction range
 	"""
-	def __init__(self, char_read: int, value, range_l, range_r):
-		super().__init__('Text length {} out of range [{}, {}]'.format(value, range_l, range_r), char_read, value, range_l, range_r)
+	def __init__(self, char_read: Union[int, str], value, range_l, range_r):
+		super().__init__('Text length {} out of range [{}, {}]'.format(value, self._get_boundary_text(range_l), self._get_boundary_text(range_r)), char_read, value, range_l, range_r)
 
 
 class IllegalEscapesUsage(IllegalArgument):
 	"""
 	The text is empty, and it's not allowed to be
 	"""
-	def __init__(self, char_read: int):
+	def __init__(self, char_read: Union[int, str]):
 		super().__init__('Illegal usage of escapes', char_read)
 
 
@@ -188,7 +221,7 @@ class UnclosedQuotedString(IllegalArgument):
 	"""
 	The text is empty, and it's not allowed to be
 	"""
-	def __init__(self, char_read: int):
+	def __init__(self, char_read: Union[int, str]):
 		super().__init__('Unclosed quoted string', char_read)
 
 
@@ -196,5 +229,18 @@ class EmptyText(IllegalArgument):
 	"""
 	The text is empty, and it's not allowed to be
 	"""
-	def __init__(self, char_read: int):
+	def __init__(self, char_read: Union[int, str]):
 		super().__init__('Empty text is not allowed', char_read)
+
+
+# Other Arguments
+
+
+class InvalidBoolean(IllegalArgument):
+	def __init__(self, char_read: Union[int, str]):
+		super().__init__('Invalid boolean', char_read)
+
+
+class InvalidEnumeration(IllegalArgument):
+	def __init__(self, char_read: Union[int, str]):
+		super().__init__('Invalid enumeration', char_read)
