@@ -1,152 +1,49 @@
 import collections
 import inspect
 from abc import ABC
-from contextlib import contextmanager
 from types import MethodType
-from typing import List, Callable, Iterable, Set, Dict, Type, Any, Union, Optional, Collection
+from typing import List, Callable, Iterable, Set, Dict, Type, Any, Union, Optional, NamedTuple
 
 from mcdreforged.command.builder import command_builder_util as utils
+from mcdreforged.command.builder.common import ParseResult, CommandContext, CommandSuggestions, CommandSuggestion
 from mcdreforged.command.builder.exception import LiteralNotMatch, UnknownCommand, UnknownArgument, CommandSyntaxError, \
 	UnknownRootArgument, RequirementNotMet, IllegalNodeOperation, \
 	CommandError
 from mcdreforged.command.command_source import CommandSource
+from mcdreforged.utils.types import MessageText
 
-SOURCE_CONTEXT_CALLBACK = Union[Callable[[], Any], Callable[[CommandSource], Any], Callable[[CommandSource, dict], Any]]
-SOURCE_CONTEXT_CALLBACK_BOOL = Union[Callable[[], bool], Callable[[CommandSource], bool], Callable[[CommandSource, dict], bool]]
-SOURCE_CONTEXT_CALLBACK_STR = Union[Callable[[], str], Callable[[CommandSource], str], Callable[[CommandSource, dict], str]]
-SOURCE_CONTEXT_CALLBACK_STR_COLLECTION = Union[Callable[[], Collection[str]], Callable[[CommandSource], Collection[str]], Callable[[CommandSource, dict], Collection[str]]]
-SOURCE_ERROR_CONTEXT_CALLBACK = Union[Callable[[], Any], Callable[[CommandSource], Any], Callable[[CommandSource, CommandError], Any], Callable[[CommandSource, CommandError, dict], Any]]
+__SOURCE_CONTEXT_CALLBACK = Union[Callable[[], Any], Callable[[CommandSource], Any], Callable[[CommandSource, dict], Any]]
+__SOURCE_CONTEXT_CALLBACK_BOOL = Union[Callable[[], bool], Callable[[CommandSource], bool], Callable[[CommandSource, dict], bool]]
+__SOURCE_CONTEXT_CALLBACK_MSG = Union[Callable[[], MessageText], Callable[[CommandSource], MessageText], Callable[[CommandSource, dict], MessageText]]
+__SOURCE_CONTEXT_CALLBACK_STR_ITERABLE = Union[Callable[[], Iterable[str]], Callable[[CommandSource], Iterable[str]], Callable[[CommandSource, dict], Iterable[str]]]
+__SOURCE_ERROR_CONTEXT_CALLBACK = Union[Callable[[], Any], Callable[[CommandSource], Any], Callable[[CommandSource, CommandError], Any], Callable[[CommandSource, CommandError, dict], Any]]
+
+RUNS_CALLBACK = __SOURCE_CONTEXT_CALLBACK
+ERROR_HANDLER_CALLBACK = __SOURCE_ERROR_CONTEXT_CALLBACK
+FAIL_MSG_CALLBACK = __SOURCE_CONTEXT_CALLBACK_MSG
+SUGGESTS_CALLBACK = __SOURCE_CONTEXT_CALLBACK_STR_ITERABLE
+REQUIRES_CALLBACK = __SOURCE_CONTEXT_CALLBACK_BOOL
 
 
-class ParseResult:
-	def __init__(self, value: Optional[Any], char_read: int):
-		self.value = value
-		self.char_read = char_read
+class _ErrorHandler(NamedTuple):
+	callback: ERROR_HANDLER_CALLBACK
+	handled: bool
 
 
-class CommandSuggestion:
-	def __init__(self, command_read: str, suggest_segment: str):
-		self.__command_read = command_read
-		self.__suggest_segment = suggest_segment
-
-	def __hash__(self):
-		return hash(self.__suggest_segment) + hash(self.__command_read) * 31
-
-	def __eq__(self, other):
-		return isinstance(other, type(self)) and self.__dict__ == other.__dict__
-
-	@property
-	def command(self) -> str:
-		return self.__command_read + self.__suggest_segment
-
-	@property
-	def existed_input(self) -> str:
-		return self.__command_read
-
-	@property
-	def suggest_input(self) -> str:
-		return self.__suggest_segment
-
-	def __str__(self):
-		return '{} -> {}'.format(self.__command_read, self.__suggest_segment)
+_ERROR_HANDLER_TYPE = Dict[Type[CommandError], _ErrorHandler]
 
 
-class CommandSuggestions(list):
-	def __init__(self, *args, **kwargs):
-		super().__init__(*args, **kwargs)
-
-		# !!MCDR plg load <file_name>
-		#                 ^
-		#               cursor
-		# "<file_name>" is the complete_hint
-		self.complete_hint = None
-
-	def extend(self, __iterable: Iterable) -> None:
-		super().extend(__iterable)
-		if isinstance(__iterable, CommandSuggestions):
-			self.complete_hint = self.complete_hint or __iterable.complete_hint
-
-
-class CommandContext(dict):
-	def __init__(self, source: CommandSource, command: str):
-		super().__init__()
-		self.__source = source
-		self.__command = command
-		self.__cursor = 0
-		self.__node_path = []  # type: List[AbstractNode]
-
-	@property
-	def source(self) -> CommandSource:
-		return self.__source
-
-	@property
-	def command(self) -> str:
-		return self.__command
-
-	@property
-	def command_read(self) -> str:
-		return self.__command[:self.__cursor]
-
-	@property
-	def command_remaining(self) -> str:
-		return self.__command[self.__cursor:]
-
-	@property
-	def cursor(self) -> int:
-		return self.__cursor
-
-	@property
-	def node_path(self) -> List['AbstractNode']:
-		return self.__node_path
-
-	@contextmanager
-	def read_command(self, current_node: 'AbstractNode', result: ParseResult, new_cursor: int):
-		"""
-		**Only used in command parsing**
-		Change the current cursor position, and store the parsing value
-		"""
-		prev_cursor = self.__cursor
-		self.__cursor = new_cursor
-		if isinstance(current_node, ArgumentNode):
-			self[current_node.get_name()] = result.value
-		try:
-			yield
-		finally:
-			self.__cursor = prev_cursor
-			if isinstance(current_node, ArgumentNode):
-				self.pop(current_node.get_name(), None)
-
-	@contextmanager
-	def enter_child(self, node: 'AbstractNode'):
-		"""
-		**Only used in command parsing**
-		Enter a command node, maintain the node_path
-		"""
-		self.__node_path.append(node)
-		try:
-			yield
-		finally:
-			self.__node_path.pop(len(self.__node_path) - 1)
-
-
-class AbstractNode:
-	class ErrorHandler:
-		def __init__(self, callback: SOURCE_ERROR_CONTEXT_CALLBACK, handled: bool):
-			self.callback = callback
-			self.handled = handled
-
-	_ERROR_HANDLER_TYPE = Dict[Type[CommandError], ErrorHandler]
-
+class AbstractNode(ABC):
 	def __init__(self):
-		self._children_literal = collections.defaultdict(list)  # type: Dict[str, List[Literal]]
-		self._children = []  # type: List[AbstractNode]
-		self._callback = None  # type: Optional[SOURCE_CONTEXT_CALLBACK]
-		self._error_handlers = {}  # type: AbstractNode._ERROR_HANDLER_TYPE
-		self._child_error_handlers = {}  # type: AbstractNode._ERROR_HANDLER_TYPE
-		self._requirement = lambda source: True  # type: SOURCE_CONTEXT_CALLBACK_BOOL
-		self._requirement_failure_message_getter = None  # type: Optional[SOURCE_CONTEXT_CALLBACK_STR]
-		self._redirect_node = None  # type: Optional[AbstractNode]
-		self._suggestion_getter = lambda: []  # type: SOURCE_CONTEXT_CALLBACK_STR_COLLECTION
+		self._children_literal: Dict[str, List[Literal]] = collections.defaultdict(list)  # mapping from literal text to related Literal nodes
+		self._children: List[AbstractNode] = []
+		self._callback: Optional[RUNS_CALLBACK] = None
+		self._error_handlers: _ERROR_HANDLER_TYPE = {}
+		self._child_error_handlers: _ERROR_HANDLER_TYPE = {}
+		self._requirement: REQUIRES_CALLBACK = lambda source: True
+		self._requirement_failure_message_getter: Optional[FAIL_MSG_CALLBACK] = None
+		self._redirect_node: Optional[AbstractNode] = None
+		self._suggestion_getter: SUGGESTS_CALLBACK = lambda: []
 
 	# --------------
 	#   Interfaces
@@ -166,7 +63,7 @@ class AbstractNode:
 			self._children.append(node)
 		return self
 
-	def runs(self, func: SOURCE_CONTEXT_CALLBACK) -> 'AbstractNode':
+	def runs(self, func: RUNS_CALLBACK) -> 'AbstractNode':
 		"""
 		Executes the given function if the command string ends here
 		:param func: A function to execute at this node which accepts maximum 2 parameters (command source and context)
@@ -175,7 +72,7 @@ class AbstractNode:
 		self._callback = func
 		return self
 
-	def requires(self, requirement: SOURCE_CONTEXT_CALLBACK_BOOL, failure_message_getter: Optional[SOURCE_CONTEXT_CALLBACK_STR] = None) -> 'AbstractNode':
+	def requires(self, requirement: REQUIRES_CALLBACK, failure_message_getter: Optional[FAIL_MSG_CALLBACK] = None) -> 'AbstractNode':
 		"""
 		Set the requirement for the command source to enter this node
 		:param requirement: A callable function which accepts maximum 2 parameters (command source and context)
@@ -200,17 +97,17 @@ class AbstractNode:
 		self._redirect_node = redirect_node
 		return self
 
-	def suggests(self, suggestion: SOURCE_CONTEXT_CALLBACK_STR_COLLECTION) -> 'AbstractNode':
+	def suggests(self, suggestion: SUGGESTS_CALLBACK) -> 'AbstractNode':
 		"""
 		Set the provider for command suggestions of this node
 		:param suggestion: A callable function which accepts maximum 2 parameters (command source and context)
-		and return a collection of str indicating the current command suggestions
+		and return an iterable of str indicating the current command suggestions
 		:rtype: AbstractNode
 		"""
 		self._suggestion_getter = suggestion
 		return self
 
-	def on_error(self, error_type: Type[CommandError], handler: SOURCE_ERROR_CONTEXT_CALLBACK, *, handled: bool = False) -> 'AbstractNode':
+	def on_error(self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> 'AbstractNode':
 		"""
 		When a command error occurs, invoke the handler
 		:param error_type: A class that is subclass of CommandError
@@ -219,10 +116,10 @@ class AbstractNode:
 		"""
 		if not issubclass(error_type, CommandError):
 			raise TypeError('error_type parameter should be a class inherited from CommandError, but class {} found'.format(error_type))
-		self._error_handlers[error_type] = self.ErrorHandler(handler, handled)
+		self._error_handlers[error_type] = _ErrorHandler(handler, handled)
 		return self
 
-	def on_child_error(self, error_type: Type[CommandError], handler: SOURCE_ERROR_CONTEXT_CALLBACK, *, handled: bool = False) -> 'AbstractNode':
+	def on_child_error(self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> 'AbstractNode':
 		"""
 		When receives a command error from one of the node's child, invoke the handler
 		:param error_type: A class that is subclass of CommandError
@@ -231,7 +128,7 @@ class AbstractNode:
 		"""
 		if not issubclass(error_type, CommandError):
 			raise TypeError('error_type parameter should be a class inherited from CommandError, but class {} found'.format(error_type))
-		self._child_error_handlers[error_type] = self.ErrorHandler(handler, handled)
+		self._child_error_handlers[error_type] = _ErrorHandler(handler, handled)
 		return self
 
 	# -------------------
@@ -264,35 +161,42 @@ class AbstractNode:
 			sig.bind(*args[:spec_args_len])  # test if using full arg length is ok
 		except TypeError:
 			raise
+
+		# make sure all passed CommandContext are copies
+		args = list(args)
+		for i in range(len(args)):
+			if isinstance(args[i], CommandContext):
+				args[i] = args[i].copy()
+
 		return callback(*args[:spec_args_len])
 
 	def __handle_error(self, error: CommandError, context: CommandContext, error_handlers: _ERROR_HANDLER_TYPE):
 		for error_type, handler in error_handlers.items():
 			if isinstance(error, error_type):
+				self.__smart_callback(handler.callback, context.source, error, context)
 				if handler.handled:
 					error.set_handled()
-				self.__smart_callback(handler.callback, context.source, error, context)
 
 	def __raise_error(self, error: CommandError, context: CommandContext):
 		self.__handle_error(error, context, self._error_handlers)
 		raise error
 
-	def _get_suggestions(self, context: CommandContext) -> List[str]:
+	def _get_suggestions(self, context: CommandContext) -> Iterable[str]:
 		return self.__smart_callback(self._suggestion_getter, context.source, context)
 
 	def _execute_command(self, context: CommandContext) -> None:
 		command = context.command  # type: str
 		try:
-			result = self.parse(context.command_remaining)
+			parse_result = self.parse(context.command_remaining)
 		except CommandSyntaxError as error:
 			error.set_parsed_command(context.command_read)
 			error.set_failed_command(context.command_read + context.command_remaining[:error.char_read])
 			self.__raise_error(error, context)
 		else:
-			next_remaining = utils.remove_divider_prefix(context.command_remaining[result.char_read:])  # type: str
+			next_remaining = utils.remove_divider_prefix(context.command_remaining[parse_result.char_read:])  # type: str
 			total_read = len(command) - len(next_remaining)  # type: int
 
-			with context.read_command(self, result, total_read):
+			with context.read_command(self, parse_result, total_read):
 				if self._requirement is not None:
 					if not self.__smart_callback(self._requirement, context.source, context):
 						getter = self._requirement_failure_message_getter or (lambda: None)
@@ -357,6 +261,8 @@ class AbstractNode:
 		# [!!aa bb cc] dd
 		# read         suggested
 		command_read_at_the_beginning = context.command_read
+		if len(context.command_remaining) == 0:
+			return self_suggestions()
 		try:
 			result = self.parse(context.command_remaining)
 		except CommandSyntaxError:
@@ -447,7 +353,7 @@ class Literal(EntryNode):
 	def _get_usage(self) -> str:
 		return '|'.join(sorted(self.literals))
 
-	def suggests(self, suggestion: SOURCE_CONTEXT_CALLBACK_STR_COLLECTION) -> 'AbstractNode':
+	def suggests(self, suggestion: SUGGESTS_CALLBACK) -> 'AbstractNode':
 		raise IllegalNodeOperation('Literal node doe not support suggests')
 
 	def parse(self, text):
