@@ -11,12 +11,14 @@ PACKAGE_NAME = 'mcdreforged'
 
 # MCDR Version Storage
 # Related: docs/source/conf.py
-VERSION = '2.0.3'       # semver (1.2.3-alpha.4)
-VERSION_PYPI = '2.0.3'  # pythonic ver (1.2.3a4)
+VERSION = '2.5.1'       # semver (1.2.3-alpha.4)
+VERSION_PYPI = '2.5.1'  # pythonic ver (1.2.3a4)
 
 GITHUB_URL = 'https://github.com/Fallen-Breath/MCDReforged'
 GITHUB_API_LATEST = 'https://api.github.com/repos/Fallen-Breath/MCDReforged/releases/latest'
+DOCUMENTION_URL = 'https://mcdreforged.readthedocs.io/'
 
+PACKAGE_PATH = os.path.realpath(os.path.join(os.path.dirname(__file__), '..'))
 LOGGING_FILE = os.path.join('logs', '{}.log'.format(NAME_SHORT))
 LANGUAGE_FILE_SUFFIX = '.yml'
 DEFAULT_LANGUAGE = 'en_us'
