@@ -1,21 +1,24 @@
 import sys
-from threading import RLock
-from typing import TYPE_CHECKING, Optional, Iterable, Any
+from threading import RLock, Lock
+from typing import TYPE_CHECKING, Optional, Iterable, Any, List
 
 from prompt_toolkit import PromptSession
 from prompt_toolkit.application import get_app
+from prompt_toolkit.buffer import CompletionState, Buffer
 from prompt_toolkit.completion import Completion, CompleteEvent, WordCompleter
 from prompt_toolkit.document import Document
 from prompt_toolkit.layout import Dimension
+# noinspection PyProtectedMember
 from prompt_toolkit.layout.menus import MultiColumnCompletionMenuControl
 from prompt_toolkit.layout.processors import Processor, TransformationInput, Transformation
 from prompt_toolkit.output.vt100 import Vt100_Output
 from prompt_toolkit.patch_stdout import StdoutProxy
 from prompt_toolkit.shortcuts import CompleteStyle
 
+from mcdreforged.command.builder.nodes.basic import CommandSuggestions
 from mcdreforged.command.command_source import CommandSource
 from mcdreforged.executor.thread_executor import ThreadExecutor
-from mcdreforged.info import Info
+from mcdreforged.info_reactor.info import Info
 from mcdreforged.permission.permission_level import PermissionLevel
 from mcdreforged.plugin.server_interface import ServerInterface
 from mcdreforged.utils import misc_util
@@ -42,55 +45,30 @@ class ConsoleHandler(ThreadExecutor):
 
 	def tick(self):
 		try:
-			text = self.console_kit.get_input()
-			if not isinstance(text, str):
+			inputs = self.console_kit.get_input()
+			if not isinstance(inputs, list):
 				raise IOError()
-			try:
-				parsed_result: Info = self.mcdr_server.server_handler_manager.get_current_handler().parse_console_command(text)
-			except:
-				self.mcdr_server.logger.exception(self.mcdr_server.tr('console_handler.parse_fail', text))
-			else:
-				if self.mcdr_server.logger.should_log_debug(DebugOption.HANDLER):
-					self.mcdr_server.logger.debug('Parsed text from {}:'.format(type(self).__name__), no_check=True)
-					for line in parsed_result.format_text().splitlines():
-						self.mcdr_server.logger.debug('	{}'.format(line), no_check=True)
-				self.mcdr_server.reactor_manager.put_info(parsed_result)
+			for text in inputs:
+				if not isinstance(text, str):
+					raise IOError()
+				try:
+					parsed_result: Info = self.mcdr_server.server_handler_manager.get_current_handler().parse_console_command(text)
+				except:
+					self.mcdr_server.logger.exception(self.mcdr_server.tr('console_handler.parse_fail', text))
+				else:
+					if self.mcdr_server.logger.should_log_debug(DebugOption.HANDLER):
+						self.mcdr_server.logger.debug('Parsed text from {}:'.format(type(self).__name__), no_check=True)
+						for line in parsed_result.format_text().splitlines():
+							self.mcdr_server.logger.debug('	{}'.format(line), no_check=True)
+					self.mcdr_server.reactor_manager.put_info(parsed_result)
 		except (KeyboardInterrupt, EOFError, SystemExit, IOError) as error:
-			if self.mcdr_server.is_server_running():
+			if not self.mcdr_server.is_mcdr_about_to_exit():
 				self.mcdr_server.logger.critical('Critical exception caught in {}: {} {}'.format(type(self).__name__, type(error).__name__, error))
-				if not self.mcdr_server.interrupt():  # not first try
-					self.mcdr_server.logger.error('Console thread stopped')
-					self.stop()
+				self.mcdr_server.interrupt()
 		except:
 			self.mcdr_server.logger.exception(self.mcdr_server.tr('console_handler.error'))
 
 
-class MCDRPromptSession(PromptSession):
-	def __init__(self, *args, **kwargs):
-		super().__init__(*args, **kwargs)
-		self.has_complete_this_line = False
-
-	def _get_default_buffer_control_height(self) -> Dimension:
-		dim = super()._get_default_buffer_control_height()
-		# When there's no complete this line, don't reverse space for the autocompletion menu
-		if not self.has_complete_this_line and self.default_buffer.complete_state is None:
-			dim = Dimension()
-		return dim
-
-	def get_complete_state_checker(self) -> Processor:
-		class CompleteStateChecker(Processor):
-			def apply_transformation(self, ti: TransformationInput) -> Transformation:
-				if len(ti.document.text) == 0:
-					session.has_complete_this_line = False
-				if buff.complete_state is not None:
-					session.has_complete_this_line = True
-				return Transformation(fragments=ti.fragments)
-
-		buff = self.default_buffer
-		session = self
-		return CompleteStateChecker()
-
-
 class ConsoleSuggestionCommandSource(CommandSource):
 	@property
 	def is_player(self) -> bool:
@@ -110,23 +88,48 @@ class ConsoleSuggestionCommandSource(CommandSource):
 		pass
 
 
-class CommandCompleter(WordCompleter):
+class CachedSuggestionProvider:
 	def __init__(self, command_manager: 'CommandManager'):
+		self.__command_manager = command_manager
+		self.__lock = Lock()
+		self.__calc_lock = Lock()
+		self.__cache_input: Optional[str] = None
+		self.__cache_suggestion: Optional[CommandSuggestions] = None
+
+	def suggest(self, input_: str) -> CommandSuggestions:
+		with self.__lock:
+			if input_ == self.__cache_input:
+				return self.__cache_suggestion
+		acq = self.__calc_lock.acquire(blocking=False)
+		if not acq:
+			return CommandSuggestions()  # empty suggestion
+		try:
+			suggestion = self.__command_manager.suggest_command(input_, ConsoleSuggestionCommandSource())
+			with self.__lock:
+				self.__cache_input = input_
+				self.__cache_suggestion = suggestion
+			return self.__cache_suggestion
+		finally:
+			self.__calc_lock.release()
+
+
+class CommandCompleter(WordCompleter):
+	def __init__(self, suggester: CachedSuggestionProvider):
 		super().__init__([], sentence=True)
-		self.command_manager = command_manager
+		self.suggester = suggester
 
 	def get_completions(self, document: Document, complete_event: CompleteEvent) -> Iterable[Completion]:
 		input_ = document.current_line_before_cursor
-		suggestions = self.command_manager.suggest_command(input_, ConsoleSuggestionCommandSource())
+		suggestions = self.suggester.suggest(input_)
 		self.words = sorted(misc_util.unique_list([suggestion.command for suggestion in suggestions]))
 		self.display_dict = dict([(suggestion.command, suggestion.suggest_input) for suggestion in suggestions])
 		return super().get_completions(document, complete_event)
 
 
 class CommandArgumentSuggester(Processor):
-	def __init__(self, command_manager: 'CommandManager'):
+	def __init__(self, suggester: CachedSuggestionProvider):
 		super().__init__()
-		self.command_manager = command_manager
+		self.suggester = suggester
 
 	def apply_transformation(self, ti: TransformationInput) -> Transformation:
 		# last line and cursor at end
@@ -134,20 +137,66 @@ class CommandArgumentSuggester(Processor):
 			buffer = ti.buffer_control.buffer
 			if not buffer.suggestion:
 				input_ = ti.document.text
-				suggestions = self.command_manager.suggest_command(input_, ConsoleSuggestionCommandSource())
+				suggestions = self.suggester.suggest(input_)
 				if suggestions.complete_hint is not None:
 					return Transformation(fragments=ti.fragments + [('class:auto-suggestion', suggestions.complete_hint)])
 		return Transformation(fragments=ti.fragments)
 
 
+class MCDRPromptSession(PromptSession):
+	def __init__(self, console_handler: ConsoleHandler):
+		self.__console_handler: ConsoleHandler = console_handler
+		# noinspection PyArgumentEqualDefault
+		super().__init__(
+			complete_in_thread=True,  # this will do the ThreadedCompleter wrapping automatically
+			complete_while_typing=True,
+			complete_style=CompleteStyle.MULTI_COLUMN,
+			reserve_space_for_menu=3
+		)
+		suggester = CachedSuggestionProvider(self.__console_handler.mcdr_server.command_manager)
+		self.completer = CommandCompleter(suggester)
+		self.input_processors = [
+			CommandArgumentSuggester(suggester),
+			self.__get_complete_state_checker()
+		]
+		self.has_complete_this_line = False
+
+	@staticmethod
+	def _has_completion_now(buffer: Buffer) -> bool:
+		# Completion calculation might be happening right now in another thread, so make a reference copy first.
+		complete_state: Optional[CompletionState] = buffer.complete_state
+		# Similarly, in addition to checking if it's None, we should checking its completions field as well
+		return complete_state is not None and complete_state.completions
+
+	def _get_default_buffer_control_height(self) -> Dimension:
+		dim = super()._get_default_buffer_control_height()
+		# When there's no complete this line, don't reverse space for the autocompletion menu
+		if not self.has_complete_this_line and not self._has_completion_now(self.default_buffer):
+			dim = Dimension()
+		return dim
+
+	def __get_complete_state_checker(self) -> Processor:
+		class __CompleteStateChecker(Processor):
+			def apply_transformation(self, ti: TransformationInput) -> Transformation:
+				if len(ti.document.text) == 0:
+					session.has_complete_this_line = False
+				if session._has_completion_now(buff):
+					session.has_complete_this_line = True
+				return Transformation(fragments=ti.fragments)
+
+		buff = self.default_buffer
+		session = self
+		return __CompleteStateChecker()
+
+
 class PromptToolkitWrapper:
 	def __init__(self, console_handler: ConsoleHandler):
-		self.console_handler = console_handler  # type: ConsoleHandler
+		self.__console_handler: ConsoleHandler = console_handler
 		self.__tr = console_handler.mcdr_server.tr
 		self.__logger = console_handler.mcdr_server.logger
 		self.pt_enabled = False
-		self.stdout_proxy = None  # type: Optional[StdoutProxy]
-		self.prompt_session = None  # type: Optional[MCDRPromptSession]
+		self.stdout_proxy: Optional[StdoutProxy] = None
+		self.prompt_session: Optional[MCDRPromptSession] = None
 		self.__real_stdout = None
 		self.__real_stderr = None
 		self.__promoting = RLock()  # more for a status check
@@ -155,7 +204,7 @@ class PromptToolkitWrapper:
 	def start_kits(self):
 		try:
 			self.__tweak_kits()
-			self.prompt_session = MCDRPromptSession()
+			self.prompt_session = MCDRPromptSession(self.__console_handler)
 			self.stdout_proxy = StdoutProxy(sleep_between_writes=0.01, raw=True)
 		except:
 			self.__logger.exception('Failed to enable advanced console, switch back to basic input')
@@ -166,6 +215,7 @@ class PromptToolkitWrapper:
 			sys.stderr = self.stdout_proxy
 			SyncStdoutStreamHandler.update_stdout()
 			self.pt_enabled = True
+			self.__logger.debug('Prompt Toolkits enabled')
 
 	@staticmethod
 	def __tweak_kits():
@@ -177,6 +227,8 @@ class PromptToolkitWrapper:
 		def min_rows_is_2(*args, **kwargs):
 			kwargs['min_rows'] = 2
 			real_ctor(*args, **kwargs)
+
+		# noinspection PyTypeChecker
 		real_ctor = MultiColumnCompletionMenuControl.__init__
 		MultiColumnCompletionMenuControl.__init__ = min_rows_is_2
 
@@ -198,21 +250,14 @@ class PromptToolkitWrapper:
 				# make sure in the console thread, prompt_session.prompt() ends
 				pass
 
-	def get_input(self) -> Optional[str]:
+	def get_input(self) -> List[str]:
 		if self.pt_enabled:
-			assert self.console_handler.is_on_thread()
-			command_manager = self.console_handler.mcdr_server.command_manager
+			assert self.__console_handler.is_on_thread()
 			with self.__promoting:
-				return self.prompt_session.prompt(
-					'> ',
-					completer=CommandCompleter(command_manager),
-					complete_while_typing=True,
-					complete_style=CompleteStyle.MULTI_COLUMN,
-					reserve_space_for_menu=3,
-					input_processors=[
-						CommandArgumentSuggester(command_manager),
-						self.prompt_session.get_complete_state_checker()
-					]
-				)
+				# It's possible for prompt-toolkit to return a string wth multiple lines when pasting (#146)
+				input_ = self.prompt_session.prompt('> ')
+				if input_ is None:
+					input_ = ''
+				return input_.splitlines()
 		else:
-			return input()
+			return [input()]
