@@ -1,21 +1,23 @@
 import re
 import time
-from typing import Optional, Any, Union, Iterable
+from typing import Optional, Any, Union, Iterable, Tuple
 
 from parse import parse
 
-from mcdreforged.info import InfoSource, Info
+from mcdreforged.info_reactor.info import InfoSource, Info
+from mcdreforged.info_reactor.server_information import ServerInformation
 from mcdreforged.utils import string_util
 
 '''
 AbstractServerHandler
  ├─ BasicHandler
- ├─ VanillaHandler
- │   ├─ BukkitHandler
- │   │   ├─ Bukkit14Handler
- │   │   └─ CatServerHandler
- │   ├─ ForgeHandler
- │   └─ Beta18Handler
+ ├─ AbstractMinecraftHandler
+ │   ├─ VanillaHandler
+ │   │   ├─ ForgeHandler
+ │   │   └─ Beta18Handler
+ │   └─ BukkitHandler
+ │       ├─ Bukkit14Handler
+ │       └─ CatServerHandler
  └─ BungeecordHandler
      └─ WaterfallHandler
 '''
@@ -39,13 +41,13 @@ class AbstractServerHandler:
 		"""
 		raise NotImplementedError()
 
-	def get_send_message_command(self, target: str, message: Any) -> Optional[str]:
+	def get_send_message_command(self, target: str, message: Any, server_information: ServerInformation) -> Optional[str]:
 		"""
 		The command to send a message to a target
 		"""
 		raise NotImplementedError()
 
-	def get_broadcast_message_command(self, message: Any) -> Optional[str]:
+	def get_broadcast_message_command(self, message: Any, server_information: ServerInformation) -> Optional[str]:
 		"""
 		The command to broadcast a message in the server
 		"""
@@ -176,6 +178,27 @@ class AbstractServerHandler:
 		"""
 		raise NotImplementedError()
 
+	def parse_server_version(self, info: Info) -> Optional[str]:
+		"""
+		Check if the info contains a server version message
+		If it is, returns server version, otherwise returns None
+
+		:param Info info: The info instance that will be checked
+		:return: The name of the player or None
+		:rtype: str or None
+		"""
+		raise NotImplementedError()
+
+	def parse_server_address(self, info: Info) -> Optional[Tuple[str, int]]:
+		"""
+		Check if the info contains the address which the server is listening on
+		If it is, returns server ip and port, otherwise returns None
+
+		:param Info info: The info instance that will be checked
+		:return: A tuple containing the ip and the port
+		"""
+		raise NotImplementedError()
+
 	def test_server_startup_done(self, info: Info) -> bool:
 		"""
 		Check if the info indicates a server startup message and return a bool
