@@ -4,19 +4,20 @@ from typing import Any, Optional
 from parse import parse
 
 from mcdreforged.handler.abstract_server_handler import AbstractServerHandler
-from mcdreforged.handler.impl.beta18_handler import Beta18Handler
-from mcdreforged.info import Info
+from mcdreforged.info_reactor.info import Info
+from mcdreforged.info_reactor.server_information import ServerInformation
 
 
 class BungeecordHandler(AbstractServerHandler):
 	def get_stop_command(self) -> str:
 		return 'end'
 
-	def get_send_message_command(self, target: str, message: Any) -> Optional[str]:
+	def get_send_message_command(self, target: str, message: Any, server_information: ServerInformation) -> Optional[str]:
 		return None
 
-	def get_broadcast_message_command(self, message: Any) -> Optional[str]:
-		return 'alertraw {}'.format(Beta18Handler.format_message(message))
+	def get_broadcast_message_command(self, message: Any, server_information: ServerInformation) -> Optional[str]:
+		from mcdreforged.handler.impl import VanillaHandler
+		return 'alertraw {}'.format(VanillaHandler.format_message(message))
 
 	@classmethod
 	def get_content_parsing_formatter(cls):
@@ -47,6 +48,17 @@ class BungeecordHandler(AbstractServerHandler):
 				return parsed['name']
 		return None
 
+	def parse_server_version(self, info: Info):
+		return None
+
+	def parse_server_address(self, info: Info):
+		# Listening on /0.0.0.0:25577
+		if not info.is_user:
+			parsed = parse('Listening on /{}:{:d}', info.content)
+			if parsed is not None:
+				return parsed[0], parsed[1]
+		return None
+
 	def test_server_startup_done(self, info: Info) -> bool:
 		# Listening on /0.0.0.0:25577
 		return not info.is_user and re.fullmatch(r'Listening on /[0-9.]+:[0-9]+', info.content) is not None
@@ -56,4 +68,4 @@ class BungeecordHandler(AbstractServerHandler):
 
 	def test_server_stopping(self, info: Info) -> bool:
 		# Closing listener [id: 0x3acae0b0, L:/0:0:0:0:0:0:0:0:25565]
-		return not info.is_user and re.fullmatch(r'Closing listener \[id: .+, L:[\d:/]+\]', info.content) is not None
+		return not info.is_user and re.fullmatch(r'Closing listener \[id: .+, L:[\d:/]+]', info.content) is not None
