@@ -5,9 +5,11 @@ import time
 import traceback
 from subprocess import Popen, PIPE, STDOUT
 from threading import Lock
-from typing import Optional
+from typing import Optional, Callable, Any
 
+import pkg_resources
 import psutil
+from ruamel.yaml import YAMLError
 
 from mcdreforged.command.command_manager import CommandManager
 from mcdreforged.config import Config
@@ -17,15 +19,17 @@ from mcdreforged.executor.task_executor import TaskExecutor
 from mcdreforged.executor.update_helper import UpdateHelper
 from mcdreforged.executor.watchdog import WatchDog
 from mcdreforged.handler.server_handler_manager import ServerHandlerManager
-from mcdreforged.info import Info
+from mcdreforged.info_reactor.info import Info
 from mcdreforged.info_reactor.info_reactor_manager import InfoReactorManager
+from mcdreforged.info_reactor.server_information import ServerInformation
 from mcdreforged.mcdr_state import ServerState, MCDReforgedState, MCDReforgedFlag
 from mcdreforged.minecraft.rcon.rcon_manager import RconManager
 from mcdreforged.permission.permission_manager import PermissionManager
 from mcdreforged.plugin.plugin_event import MCDRPluginEvents
 from mcdreforged.plugin.plugin_manager import PluginManager
 from mcdreforged.plugin.server_interface import ServerInterface
-from mcdreforged.translation_manager import TranslationManager
+from mcdreforged.preference.preference_manager import PreferenceManager
+from mcdreforged.translation.translation_manager import TranslationManager
 from mcdreforged.utils import file_util
 from mcdreforged.utils.exception import IllegalCallError, ServerStopped, ServerStartError, IllegalStateError
 from mcdreforged.utils.logger import DebugOption, MCDReforgedLogger, MCColoredFormatter
@@ -37,10 +41,11 @@ class MCDReforgedServer:
 
 	def __init__(self, *, generate_default_only: bool = False, initialize_environment: bool = False):
 		"""
-		:param generate_default_only: If set to true, MCDR will only generate the default configure and permission files
+		:param generate_default_only: If set to true, MCDR will only generate the default configuration and permission files
 		"""
 		self.mcdr_state = MCDReforgedState.INITIALIZING
 		self.server_state = ServerState.STOPPED
+		self.server_information = ServerInformation()
 		self.process = None  # type: Optional[PIPE]
 		self.flags = MCDReforgedFlag.NONE
 		self.starting_server_lock = Lock()  # to prevent multiple start_server() call
@@ -51,7 +56,7 @@ class MCDReforgedServer:
 		self.decoding_method = None  # type: Optional[str]
 
 		# --- Constructing fields --- #
-		self.logger = MCDReforgedLogger(self)
+		self.logger = MCDReforgedLogger()
 		self.config = Config(self.logger)
 		self.permission_manager = PermissionManager(self)
 		self.basic_server_interface = ServerInterface(self)
@@ -65,6 +70,9 @@ class MCDReforgedServer:
 		self.reactor_manager = InfoReactorManager(self)
 		self.command_manager = CommandManager(self)
 		self.plugin_manager = PluginManager(self)
+		self.preference_manager = PreferenceManager(self)
+
+		self.__check_environment()
 
 		# --- Input arguments "generate_default_only" processing --- #
 		if generate_default_only:
@@ -86,20 +94,32 @@ class MCDReforgedServer:
 		# Check if there's any file missing
 		# If there's any, MCDR environment might not be probably setup
 		file_missing = False
-		try:
-			# loads config, language, handlers
-			# config change will lead to creating plugin folders
-			self.load_config(allowed_missing_file=False, echo=not initialize_environment)
-		except FileNotFoundError:
-			self.logger.error('Configure is missing')
-			file_missing = True
-		try:
-			self.permission_manager.load_permission_file(allowed_missing_file=False)
-		except FileNotFoundError:
-			self.logger.error('Permission file is missing')
-			file_missing = True
+
+		def load(kind: str, func: Callable[[], Any]) -> bool:
+			nonlocal file_missing
+			try:
+				func()
+			except FileNotFoundError:
+				self.logger.error('{} is missing'.format(kind.title()))
+				file_missing = True
+			except YAMLError as e:
+				self.logger.error('Failed to load {}: {}'.format(kind, type(e).__name__))
+				for line in str(e).splitlines():
+					self.logger.error(line)
+				return False
+			else:
+				return True
+
+		# load_config: config, language, handlers, plugin directories, reactors, handlers
+		# load_permission_file: permission
+		# config change will lead to creating plugin folders
+		loading_success = \
+			load('configuration', lambda: self.load_config(allowed_missing_file=False, echo=not initialize_environment)) and \
+			load('permission', lambda: self.permission_manager.load_permission_file(allowed_missing_file=False))
 		if file_missing:
-			self.on_file_missing()
+			self.__on_file_missing()
+			return
+		if not loading_success:
 			return
 
 		# MCDR environment has been setup, so continue creating default folders and loading stuffs
@@ -116,7 +136,22 @@ class MCDReforgedServer:
 		except:
 			pass
 
-	def on_file_missing(self):
+	def __check_environment(self):
+		"""
+		Some checks at initialization
+		In dev environment, you can use setup.py to create `mcdreforged.egg-info/` so the package check can pass
+		"""
+		mcdr_pkg = core_constant.PACKAGE_NAME  # should be "mcdreforged"
+		try:
+			pkg_resources.require(mcdr_pkg)
+		except pkg_resources.ResolutionError:
+			self.logger.warning('It looks like you\'re launching MCDR from source, since {} is not found in python packages'.format(mcdr_pkg))
+			self.logger.warning('In this way, the plugin system might not work correctly')
+			self.logger.warning('In a production environment, you should install {} from PyPI, see document ({}) for more information'.format(mcdr_pkg, core_constant.DOCUMENTION_URL))
+			self.logger.warning('MCDR will launch after 20 seconds...')
+			time.sleep(20)
+
+	def __on_file_missing(self):
 		self.logger.info('Looks like MCDR is not initialized at current directory {}'.format(os.getcwd()))
 		self.logger.info('Use "python -m {} init" to initialize MCDR first'.format(core_constant.PACKAGE_NAME))
 
@@ -124,7 +159,10 @@ class MCDReforgedServer:
 	#         Translate
 	# --------------------------
 
-	def tr(self, translation_key: str, *args, language: Optional[str] = None, fallback_language: Optional[str] = None, allow_failure=True, **kwargs) -> MessageText:
+	def get_language(self) -> str:
+		return self.translation_manager.language
+
+	def tr(self, translation_key: str, *args, language: Optional[str] = None, allow_failure=True, **kwargs) -> MessageText:
 		"""
 		Return a translated text corresponded to the translation key and format the text with given args
 		If args contains RText element, then the result will be a RText, otherwise the result will be a regular str
@@ -132,7 +170,6 @@ class MCDReforgedServer:
 		:param translation_key: The key of the translation
 		:param args: The args to be formatted
 		:param language: Specific language to be used in this translation, or the language that MCDR is using will be used
-		:param fallback_language: Fallback language used when the current language translation not found
 		:param allow_failure: If set to false, a KeyError will be risen if the translation key is not recognized
 		:param kwargs: The kwargs to be formatted
 		"""
@@ -140,7 +177,7 @@ class MCDReforgedServer:
 			translation_key, args, kwargs,
 			allow_failure=allow_failure,
 			language=language,
-			fallback_language=fallback_language,
+			fallback_language=self.translation_manager.language,
 			plugin_translations=self.plugin_manager.registry_storage.translations
 		)
 
@@ -194,6 +231,15 @@ class MCDReforgedServer:
 		self.command_manager.clear_command()
 		self.plugin_manager.registry_storage.export_commands(self.command_manager.register_command)
 
+	# ---------------------------
+	#      General Setters
+	# ---------------------------
+	# for field read-only access, simply use directly reference
+	# but for field writing operation, use setters
+
+	def set_task_executor(self, new_task_executor: TaskExecutor):
+		self.task_executor = new_task_executor
+
 	# ---------------------------
 	#   State Getters / Setters
 	# ---------------------------
@@ -316,20 +362,20 @@ class MCDReforgedServer:
 		else:
 			raise IllegalCallError("Server process has already been terminated")
 
-	def interrupt(self):
+	def interrupt(self) -> bool:
 		"""
 		Interrupt MCDR
 		The first call will softly stop the server and the later calls will kill the server
 		Return if it's the first try
-		:rtype: bool
 		"""
-		self.logger.info('Interrupting, first strike = {}'.format(not self.is_interrupt()))
-		self.stop(forced=self.is_interrupt())
-		first_try = not self.is_interrupt()
+		first_interrupt = not self.is_interrupt()
+		self.logger.info('Interrupting, first strike = {}'.format(first_interrupt))
+		if self.is_server_running():
+			self.stop(forced=not first_interrupt)
 		self.with_flag(MCDReforgedFlag.INTERRUPT)
-		return first_try
+		return first_interrupt
 
-	def stop(self, forced=False):
+	def stop(self, forced: bool = False):
 		"""
 		Stop the server
 
@@ -361,6 +407,7 @@ class MCDReforgedServer:
 		self.set_server_state(ServerState.RUNNING)
 		self.with_flag(MCDReforgedFlag.EXIT_AFTER_STOP)  # Set after server state is set to RUNNING, or MCDR might have a chance to exit if the server is started by other thread
 		self.logger.info(self.tr('mcdr_server.start_server.pid_info', self.process.pid))
+		self.reactor_manager.on_server_start()
 		self.plugin_manager.dispatch_event(MCDRPluginEvents.SERVER_START, ())
 
 	def __on_server_stop(self):
@@ -377,6 +424,7 @@ class MCDReforgedServer:
 		self.process = None
 		self.set_server_state(ServerState.STOPPED)
 		self.remove_flag(MCDReforgedFlag.SERVER_STARTUP | MCDReforgedFlag.SERVER_RCON_READY)  # removes this two
+		self.reactor_manager.on_server_stop()
 		self.plugin_manager.dispatch_event(MCDRPluginEvents.SERVER_STOP, (return_code,), block=True)
 
 		if self.is_interrupt():
@@ -469,6 +517,7 @@ class MCDReforgedServer:
 	def __on_mcdr_start(self):
 		self.watch_dog.start()
 		self.task_executor.start()
+		self.preference_manager.load_preferences()
 		self.plugin_manager.register_permanent_plugins()
 		self.task_executor.execute_on_thread(self.load_plugins, block=True)
 		self.plugin_manager.dispatch_event(MCDRPluginEvents.MCDR_START, ())
