@@ -1,6 +1,9 @@
 import collections
 from enum import Enum
-from typing import Optional, Dict, List
+from typing import Optional, Dict, List, Union
+
+
+PermissionParam = Union[str, int]
 
 
 class PermissionLevelItem:
@@ -22,17 +25,17 @@ class PermissionLevelItem:
 
 class PermissionLevel:
 	class __Storage(Enum):
-		GUEST	= PermissionLevelItem('guest', 0)
-		USER	= PermissionLevelItem('user', 1)
-		HELPER	= PermissionLevelItem('helper', 2)
-		ADMIN	= PermissionLevelItem('admin', 3)
-		OWNER	= PermissionLevelItem('owner', 4)
+		GUEST_	= PermissionLevelItem('guest', 0)
+		USER_	= PermissionLevelItem('user', 1)
+		HELPER_	= PermissionLevelItem('helper', 2)
+		ADMIN_	= PermissionLevelItem('admin', 3)
+		OWNER_	= PermissionLevelItem('owner', 4)
 
-	GUEST = __Storage.GUEST.value.level
-	USER = __Storage.USER.value.level
-	HELPER = __Storage.HELPER.value.level
-	ADMIN = __Storage.ADMIN.value.level
-	OWNER = __Storage.OWNER.value.level
+	GUEST = __Storage.GUEST_.value.level
+	USER = __Storage.USER_.value.level
+	HELPER = __Storage.HELPER_.value.level
+	ADMIN = __Storage.ADMIN_.value.level
+	OWNER = __Storage.OWNER_.value.level
 
 	INSTANCES = [item.value for item in __Storage]  # type: List[PermissionLevelItem]
 	LEVELS = [inst.level for inst in INSTANCES]  # type: List[int]
@@ -55,7 +58,7 @@ class PermissionLevel:
 			raise ValueError('Value {} out of range [{}, {}]'.format(level, cls.MINIMUM_LEVEL, cls.MAXIMUM_LEVEL))
 
 	@classmethod
-	def from_value(cls, value):
+	def from_value(cls, value: PermissionParam):
 		"""
 		Convert any type of permission level into int value. Examples:
 			'guest'	-> 0
@@ -82,7 +85,7 @@ class PermissionLevel:
 		return level
 
 	@classmethod
-	def get_level(cls, value) -> Optional[PermissionLevelItem]:
+	def get_level(cls, value: PermissionParam) -> Optional[PermissionLevelItem]:
 		"""
 		Fail-proof version of from_value
 		"""
