@@ -9,7 +9,7 @@ from typing import Callable, Dict, Optional, Any, Tuple, List, TYPE_CHECKING, De
 
 from mcdreforged.constants import core_constant, plugin_constant
 from mcdreforged.plugin import plugin_factory
-from mcdreforged.plugin.builtin.mcdreforged_plugin import MCDReforgedPlugin
+from mcdreforged.plugin.builtin.mcdreforged_plugin.mcdreforged_plugin import MCDReforgedPlugin
 from mcdreforged.plugin.builtin.python_plugin import PythonPlugin
 from mcdreforged.plugin.meta.dependency_walker import DependencyWalker
 from mcdreforged.plugin.operation_result import PluginOperationResult, SingleOperationResult
@@ -107,10 +107,14 @@ class PluginManager:
 				self.__tls.pop(self.TLS_PLUGIN_KEY)
 
 	def contains_plugin_file(self, file_path: str) -> bool:
+		"""
+		Check if the given path corresponds to an already loaded plugin
+		"""
 		return os.path.abspath(file_path) in self.plugin_file_path
 
 	def contains_plugin_id(self, plugin_id: str) -> bool:
 		"""
+		Check if the given plugin id represents a loaded plugin
 		Includes permanent plugins
 		"""
 		return plugin_id in self.plugins
@@ -244,21 +248,35 @@ class PluginManager:
 	#   Regular Plugin Collector & Handlers
 	# ---------------------------------------
 
-	def __collect_and_process_new_plugins(self, filter: Callable[[str], bool]) -> SingleOperationResult:
-		result = SingleOperationResult()
+	def __collect_possible_plugin_file_paths(self) -> List[str]:
+		paths = []
 		for plugin_directory in self.plugin_directories:
 			if os.path.isdir(plugin_directory):
 				for file in os.listdir(plugin_directory):
 					file_path = os.path.join(plugin_directory, file)
-					if plugin_factory.maybe_plugin(file_path):
-						if not self.contains_plugin_file(file_path) and filter(file_path):
-							plugin = self.__load_plugin(file_path)
-							if plugin is None:
-								result.fail(file_path)
-							else:
-								result.succeed(plugin)
+					if plugin_factory.is_plugin(file_path):
+						paths.append(file_path)
 			else:
 				self.logger.warning('Plugin directory "{}" not found'.format(plugin_directory))
+		return paths
+
+	def __collect_and_process_new_plugins(self, filter: Callable[[str], bool], *, possible_paths: Optional[List[str]] = None) -> SingleOperationResult:
+		"""
+		:param filter: A str predicate function for testing if the plugin file path is acceptable
+		:param possible_paths: Optional. If you have already done self.__collect_possible_plugin_file_paths() before,
+		you can pass the previous result as the argument to reuse that, so less time cost
+		"""
+		if possible_paths is None:
+			possible_paths = self.__collect_possible_plugin_file_paths()
+
+		result = SingleOperationResult()
+		for file_path in possible_paths:
+			if not self.contains_plugin_file(file_path) and filter(file_path):
+				plugin = self.__load_plugin(file_path)
+				if plugin is None:
+					result.fail(file_path)
+				else:
+					result.succeed(plugin)
 		return result
 
 	def __collect_and_remove_plugins(self, filter: Callable[[RegularPlugin], bool], specific: Optional[RegularPlugin] = None) -> SingleOperationResult:
@@ -306,17 +324,18 @@ class PluginManager:
 	# Current behavior for plugin operations
 	# 1. Actual plugin operations
 	#   For plugin refresh
-	#     1. Unload plugins whose file is removed
-	#     2. Load new plugins
-	#     3. Reload existed (and matches filter) plugins whose state is ready. if reload fail unload it
+	#      1. Unload plugins whose file is removed
+	#      2. Load new plugins
+	#      3. Reload existed (and matches filter) plugins whose state is ready. if reload fail unload it
 	#
 	#   For single plugin operation
-	#     1. Enable / disable plugin
+	#      1. Enable / disable plugin
 	#
 	# 2. Plugin Processing  (method __post_plugin_process)
-	#     1. Check dependencies, unload plugins that has dependencies not satisfied
-	#     2. Call on_load for new / reloaded plugins by topo order
-	#     3. Call on_unload for unloaded plugins
+	#      1. Check dependencies, unload plugins that has dependencies not satisfied
+	#      2. Call on_unload for unloaded plugins
+	#      3. Call on_load for new / reloaded plugins by topo order
+	#    Dispatch PLUGIN_UNLOADED before PLUGIN_LOADED, so file renamed plugin keeps the same event order (unload -> load) with reloaded plugin
 
 	def __post_plugin_process(self, load_result=None, unload_result=None, reload_result=None):
 		"""
@@ -342,24 +361,37 @@ class PluginManager:
 		self.registry_storage.clear()  # in case plugin invokes dispatch_event during on_load. dont let them trigger listeners
 
 		newly_loaded_plugins = {*load_result.success_list, *reload_result.success_list}
+		to_be_removed_plugins = unload_result.success_list + unload_result.failed_list + reload_result.failed_list + dependency_check_result.failed_list
+
+		# collect removed plugin module instance
+		removed_plugins_module_instance = {}
+		for plugin in to_be_removed_plugins:
+			if isinstance(plugin, RegularPlugin):
+				removed_plugins_module_instance[plugin.get_id()] = plugin.entry_module_instance
+
+		# get ready
 		for plugin in dependency_check_result.success_list:
 			if plugin in newly_loaded_plugins:
 				plugin.ready()
 
-		for plugin in dependency_check_result.success_list:
-			if plugin in newly_loaded_plugins:
-				if isinstance(plugin, RegularPlugin):
-					plugin.receive_event(MCDRPluginEvents.PLUGIN_LOADED, (plugin.old_entry_module_instance,))
-
-		for plugin in unload_result.success_list + unload_result.failed_list + reload_result.failed_list + dependency_check_result.failed_list:
+		# PLUGIN_UNLOADED event
+		for plugin in to_be_removed_plugins:
 			plugin.assert_state({PluginState.UNLOADING})
 			# plugins might just be newly loaded but failed on dependency check, dont dispatch event to them
 			if plugin not in newly_loaded_plugins:
 				plugin.receive_event(MCDRPluginEvents.PLUGIN_UNLOADED, ())
-			# plugin.receive_event(MCDRPluginEvents.PLUGIN_REMOVED, ())
 			plugin.remove()
 
-		# they should be
+		# PLUGIN_LOADED event
+		for plugin in dependency_check_result.success_list:
+			if plugin in newly_loaded_plugins:
+				if isinstance(plugin, RegularPlugin):
+					old_entry_module_instance = plugin.old_entry_module_instance
+					if old_entry_module_instance is None:
+						old_entry_module_instance = removed_plugins_module_instance.get(plugin.get_id(), None)
+					plugin.receive_event(MCDRPluginEvents.PLUGIN_LOADED, (old_entry_module_instance,))
+
+		# they should all be ready
 		for plugin in self.get_regular_plugins():
 			plugin.assert_state({PluginState.READY})
 
@@ -377,8 +409,10 @@ class PluginManager:
 		self.mcdr_server.on_plugin_registry_changed()
 
 	def __refresh_plugins(self, reload_filter: Callable[[RegularPlugin], bool]):
-		unload_result = self.__collect_and_remove_plugins(lambda plugin: not plugin.plugin_exists())
-		load_result = self.__collect_and_process_new_plugins(lambda fp: True)
+		possible_paths = self.__collect_possible_plugin_file_paths()
+		possible_paths_set = set(possible_paths)
+		unload_result = self.__collect_and_remove_plugins(lambda plugin: not plugin.plugin_exists() or plugin.plugin_path not in possible_paths_set)
+		load_result = self.__collect_and_process_new_plugins(lambda fp: True, possible_paths=possible_paths)
 		reload_result = self.__reload_ready_plugins(reload_filter)
 		self.__post_plugin_process(load_result, unload_result, reload_result)
 
@@ -408,7 +442,7 @@ class PluginManager:
 		with self.__mani_lock:
 			self.logger.info(self.mcdr_server.tr('plugin_manager.enable_plugin.entered', file_path))
 			new_file_path = string_util.remove_suffix(file_path, plugin_constant.DISABLED_PLUGIN_FILE_SUFFIX)
-			if plugin_factory.maybe_plugin(file_path, allow_disabled=True):
+			if plugin_factory.is_disabled_plugin(file_path):
 				os.rename(file_path, new_file_path)
 				self.load_plugin(new_file_path)
 
@@ -441,7 +475,7 @@ class PluginManager:
 		for listener in self.registry_storage.event_listeners.get(event.id, []):
 			self.trigger_listener(listener, args)
 
-	def dispatch_event(self, event: MCDREvent, args: Tuple[Any, ...], *, on_executor_thread=True, block: bool = False, timeout: Optional[float] = None):
+	def dispatch_event(self, event: MCDREvent, args: Tuple[Any, ...], *, on_executor_thread: bool = True, block: bool = False, timeout: Optional[float] = None):
 		"""
 		Event dispatching interface
 		"""
