@@ -6,24 +6,29 @@ from typing import Callable, TYPE_CHECKING, Tuple, Any, Union, Optional, List, I
 
 from mcdreforged.command.builder.nodes.basic import Literal
 from mcdreforged.command.command_source import CommandSource, PluginCommandSource
-from mcdreforged.constants import plugin_constant, core_constant
-from mcdreforged.info import Info
+from mcdreforged.constants import plugin_constant
+from mcdreforged.info_reactor.info import Info
+from mcdreforged.info_reactor.server_information import ServerInformation
 from mcdreforged.mcdr_state import MCDReforgedFlag
-from mcdreforged.permission.permission_level import PermissionLevel
+from mcdreforged.permission.permission_level import PermissionLevel, PermissionParam
+from mcdreforged.plugin import plugin_factory
 from mcdreforged.plugin.meta.metadata import Metadata
 from mcdreforged.plugin.operation_result import SingleOperationResult, PluginOperationResult
 from mcdreforged.plugin.plugin_event import EventListener, LiteralEvent, PluginEvent, MCDRPluginEvents
 from mcdreforged.plugin.plugin_registry import DEFAULT_LISTENER_PRIORITY, HelpMessage
 from mcdreforged.plugin.type.multi_file_plugin import MultiFilePlugin
 from mcdreforged.plugin.type.plugin import AbstractPlugin
-from mcdreforged.utils import misc_util
+from mcdreforged.preference.preference_manager import PreferenceItem
+from mcdreforged.translation.translation_text import RTextMCDRTranslation
+from mcdreforged.utils import misc_util, file_util
 from mcdreforged.utils.exception import IllegalCallError
 from mcdreforged.utils.logger import MCDReforgedLogger, DebugOption
 from mcdreforged.utils.serializer import Serializable
-from mcdreforged.utils.types import MessageText, TranslationKeyDictRich, TranslationKeyDict
+from mcdreforged.utils.types import MessageText, TranslationKeyDictRich, TranslationKeyDictNested
 
 if TYPE_CHECKING:
 	from mcdreforged.mcdr_server import MCDReforgedServer
+	from mcdreforged.handler.abstract_server_handler import AbstractServerHandler
 	from mcdreforged.plugin.plugin_manager import PluginManager
 	from mcdreforged.plugin.type.regular_plugin import RegularPlugin
 
@@ -49,7 +54,7 @@ class ServerInterface:
 
 	@functools.lru_cache(maxsize=512, typed=True)
 	def __get_logger(self, plugin_id: str):
-		return MCDReforgedLogger(self._mcdr_server, plugin_id)
+		return MCDReforgedLogger(plugin_id)
 
 	@property
 	def logger(self) -> MCDReforgedLogger:
@@ -73,7 +78,7 @@ class ServerInterface:
 		"""
 		return cls.__global_instance
 
-	def tr(self, translation_key: str, *args, language: Optional[str] = None, fallback_language: str = core_constant.DEFAULT_LANGUAGE, **kwargs) -> MessageText:
+	def tr(self, translation_key: str, *args, language: Optional[str] = None, **kwargs) -> MessageText:
 		"""
 		Return a translated text corresponded to the translation key and format the text with given args and kwargs
 		If args or kwargs contains RText element, then the result will be a RText, otherwise the result will be a regular str
@@ -81,10 +86,22 @@ class ServerInterface:
 		:param translation_key: The key of the translation
 		:param args: The args to be formatted
 		:param language: Specific language to be used in this translation, or the language that MCDR is using will be used
-		:param fallback_language: Fallback language used when the current language translation not found
 		:param kwargs: The kwargs to be formatted
 		"""
-		return self._mcdr_server.tr(translation_key, *args, language=language, fallback_language=fallback_language, **kwargs)
+		return self._mcdr_server.tr(translation_key, *args, language=language, **kwargs)
+
+	def rtr(self, translation_key: str, *args, **kwargs) -> RTextMCDRTranslation:
+		"""
+		Return a RText derived component RTextMCDRTranslation, that only translates itself right before displaying or serializing
+		Using this method instead of tr() allows you to display your texts in user's preferred language automatically
+		Of course you can construct RTextMCDRTranslation yourself instead of using this method if you want
+		:param translation_key: The key of the translation
+		:param args: The args to be formatted
+		:param kwargs: The kwargs to be formatted
+		"""
+		text = RTextMCDRTranslation(translation_key, *args, **kwargs)
+		text.set_translator(self.tr)  # not that necessary tbh, just in case self.tr != ServerInterface.get_instance().tr somehow
+		return text
 
 	def as_basic_server_interface(self) -> 'ServerInterface':
 		"""
@@ -165,8 +182,9 @@ class ServerInterface:
 	def set_exit_after_stop_flag(self, flag_value: bool) -> None:
 		"""
 		Set the flag that indicating if MCDR should exit when the server has stopped
-		If set to true, after the server stops MCDR will exit,
-		otherwise MCDR will just keep running
+		If set to true, after the server stops MCDR will exit, otherwise (set to false) MCDR will just keep running
+		The flag value will be set to true everything when the server starts
+		The flag value is displayed in line 5 in command `!!MCDR status`
 		"""
 		if flag_value:
 			self._mcdr_server.with_flag(MCDReforgedFlag.EXIT_AFTER_STOP)
@@ -202,6 +220,14 @@ class ServerInterface:
 			return self._mcdr_server.process.pid
 		return None
 
+	def get_server_information(self) -> ServerInformation:
+		"""
+		Return a ServerInformation object indicating the information of the current server, interred from the output of the server
+		Field(s) might be None if the server is offline, or the related information has not been parsed
+		:return: a ServerInformation object
+		"""
+		return self._mcdr_server.server_information.copy()
+
 	# ------------------------
 	#     Text Interaction
 	# ------------------------
@@ -215,6 +241,10 @@ class ServerInterface:
 		self.logger.debug('Sending command "{}"'.format(text), option=DebugOption.PLUGIN)
 		self._mcdr_server.send(text, encoding=encoding)
 
+	@property
+	def __server_handler(self) -> 'AbstractServerHandler':
+		return self._mcdr_server.server_handler_manager.get_current_handler()
+
 	def tell(self, player: str, text: MessageText, *, encoding: Optional[str] = None) -> None:
 		"""
 		Use command like /tellraw to send the message to the specific player
@@ -222,7 +252,8 @@ class ServerInterface:
 		:param text: the message you want to send to the player
 		:param encoding: The encoding method for the text
 		"""
-		command = self._mcdr_server.server_handler_manager.get_current_handler().get_send_message_command(player, text)
+		with RTextMCDRTranslation.language_context(self._mcdr_server.preference_manager.get_preferred_language(player)):
+			command = self.__server_handler.get_send_message_command(player, text, self.get_server_information())
 		if command is not None:
 			self.execute(command, encoding=encoding)
 
@@ -232,7 +263,7 @@ class ServerInterface:
 		:param text: the message you want to send
 		:param encoding: The encoding method for the text
 		"""
-		command = self._mcdr_server.server_handler_manager.get_current_handler().get_broadcast_message_command(text)
+		command = self.__server_handler.get_broadcast_message_command(text, self.get_server_information())
 		if command is not None:
 			self.execute(command, encoding=encoding)
 
@@ -305,6 +336,30 @@ class ServerInterface:
 		"""
 		return [plugin.get_id() for plugin in self._mcdr_server.plugin_manager.get_regular_plugins()]
 
+	def __get_files_in_plugin_directories(self) -> List[str]:
+		result = []
+		for plugin_directory in self._mcdr_server.plugin_manager.plugin_directories:
+			result.extend(file_util.list_all(plugin_directory))
+		return result
+
+	def get_unloaded_plugin_list(self) -> List[str]:
+		"""
+		Return a list containing all unloaded plugin file path like ["plugins/MyPlugin.mcdr"]
+		"""
+		return list(filter(
+			lambda file_path: not self._mcdr_server.plugin_manager.contains_plugin_file(file_path) and plugin_factory.is_plugin(file_path),
+			self.__get_files_in_plugin_directories()
+		))
+
+	def get_disabled_plugin_list(self) -> List[str]:
+		"""
+		Return a list containing all disabled plugin file path like ["plugins/MyPlugin.mcdr.disabled"]
+		"""
+		return list(filter(
+			lambda file_path: plugin_factory.is_disabled_plugin(file_path),
+			self.__get_files_in_plugin_directories()
+		))
+
 	def get_all_metadata(self) -> Dict[str, Metadata]:
 		"""
 		Return a dict containing metadatas of all loaded plugin with (plugin_id, metadata) as key-value pair
@@ -450,7 +505,7 @@ class ServerInterface:
 		else:
 			raise TypeError('Unsupported permission level querying for type {}'.format(type(obj)))
 
-	def set_permission_level(self, player: str, value: Union[int, str]) -> None:
+	def set_permission_level(self, player: str, value: PermissionParam) -> None:
 		"""
 		Set the permission level of the given player
 		:param player: The name of the player that you want to set his/her permission level
@@ -461,7 +516,7 @@ class ServerInterface:
 		level = PermissionLevel.get_level(value)
 		if level is None:
 			raise TypeError('Parameter level needs to be a permission related value')
-		self._mcdr_server.permission_manager.set_permission_level(player, value)
+		self._mcdr_server.permission_manager.set_permission_level(player, level)
 
 	# ------------------------
 	#         Command
@@ -487,6 +542,20 @@ class ServerInterface:
 		misc_util.check_type(source, CommandSource)
 		self._mcdr_server.command_manager.execute_command(command, source)
 
+	# ------------------------
+	#        Preference
+	# ------------------------
+
+	def get_preference(self, obj: Union[str, CommandSource]) -> PreferenceItem:
+		"""
+		Return the MCDR preference of the given object. The object can be a str indicating the name of a player, or a
+		command source. For command source, only PlayerCommandSource and ConsoleCommandSource are supported
+		:param obj: The object to querying preference
+		:raise: TypeError, if the type of the given object is not supported for preference querying
+		"""
+		pref = self._mcdr_server.preference_manager.get_preference(obj, strict_type_check=True)
+		return PreferenceItem.deserialize(pref.serialize())  # make a copy
+
 	# ------------------------
 	#           Misc
 	# ------------------------
@@ -543,7 +612,7 @@ class PluginServerInterface(ServerInterface):
 	def logger(self) -> MCDReforgedLogger:
 		if self.__logger_for_plugin is None:
 			try:
-				self.__logger_for_plugin = MCDReforgedLogger(self._mcdr_server, self.__plugin.get_id())
+				self.__logger_for_plugin = MCDReforgedLogger(self.__plugin.get_id())
 			except:
 				return super().logger
 		return self.__logger_for_plugin
@@ -593,11 +662,11 @@ class PluginServerInterface(ServerInterface):
 		"""
 		self.__plugin.register_help_message(HelpMessage(self.__plugin, prefix, message, permission))
 
-	def register_translation(self, language: str, mapping: TranslationKeyDict) -> None:
+	def register_translation(self, language: str, mapping: TranslationKeyDictNested) -> None:
 		"""
 		Register a translation mapping for a specific language for the current plugin
 		:param language: The language of this translation
-		:param mapping: A dict which maps translation keys into translated text
+		:param mapping: A dict which maps translation keys into translated text. The translation key could be expressed as node name which under root node or the path of a nested multi-level nodes
 		"""
 		self.__plugin.register_translation(language, mapping)
 
@@ -636,12 +705,13 @@ class PluginServerInterface(ServerInterface):
 
 	def load_config_simple(
 			self, file_name: str = 'config.json', default_config: Optional = None, *,
-			in_data_folder: bool = True, echo_in_console: bool = True, source_to_reply: Optional[CommandSource] = None, target_class: Optional[Type[SerializableType]] = None
+			in_data_folder: bool = True, echo_in_console: bool = True, source_to_reply: Optional[CommandSource] = None,
+			target_class: Optional[Type[SerializableType]] = None, encoding: str = 'utf8'
 		) -> Union[dict, SerializableType]:
 		"""
 		A simple method to load a dict or Serializable type config from a json file
 		Default config is supported. Missing key-values in the loaded config object will be filled using the default config
-		:param file_name: The name of the config file
+		:param file_name: The name of the config file. It can also be a path to the config file
 		:param default_config: A dict contains the default config. It's required when the config file is missing,
 		or exception will be risen. If target_class is given and default_config is missing, the default values in target_class
 		will be used when the config file is missing
@@ -650,23 +720,24 @@ class PluginServerInterface(ServerInterface):
 		:param source_to_reply: The command source for replying logging messages
 		:param target_class: A class derived from Serializable. When specified the loaded config data will be deserialized
 		to a instance of target_class which will be returned as return value
+		:param encoding: The encoding method to read the config file
 		:return: A dict contains the loaded and processed config
 		"""
-		def log(msg):
+		def log(msg: str):
 			if isinstance(source_to_reply, CommandSource):
 				source_to_reply.reply(msg)
-			if echo_in_console:
+			# don't do double-echo if the source is a console command source
+			if echo_in_console and not (source_to_reply is not None and source_to_reply.is_console):
 				self.logger.info(msg)
 
 		if target_class is not None:
-			misc_util.check_class(target_class, Serializable)
-			target_class: Serializable
+			target_class: Type[Serializable]
 			if default_config is None:
-				default_config = target_class.deserialize({}).serialize()
+				default_config = target_class.get_default().serialize()
 		config_file_path = os.path.join(self.get_data_folder(), file_name) if in_data_folder else file_name
 		needs_save = False
 		try:
-			with open(config_file_path) as file_handler:
+			with open(config_file_path, encoding=encoding) as file_handler:
 				read_data: dict = json.load(file_handler)
 		except Exception as e:
 			if default_config is not None:  # use default config
@@ -696,24 +767,33 @@ class PluginServerInterface(ServerInterface):
 				log(self._mcdr_server.tr('server_interface.load_config_simple.failed', e))
 		else:
 			# remove unexpected keys
-			for key in list(result_config.keys()):
-				if key not in default_config:
-					result_config.pop(key)
+			if default_config is not None:
+				for key in list(result_config.keys()):
+					if key not in default_config:
+						result_config.pop(key)
 		if needs_save:
 			self.save_config_simple(result_config, file_name=file_name, in_data_folder=in_data_folder)
 		return result_config
 
-	def save_config_simple(self, config: Union[dict, Serializable], file_name: str = 'config.json', *, in_data_folder: bool = True) -> None:
+	def save_config_simple(
+			self, config: Union[dict, Serializable], file_name: str = 'config.json', *,
+			in_data_folder: bool = True, encoding: str = 'utf8'
+	) -> None:
 		"""
 		A simple method to save your dict or Serializable type config as a json file
 		:param config: The config instance to be saved
-		:param file_name: The name of the config file
+		:param file_name: The name of the config file. It can also be a path to the config file
 		:param in_data_folder: If True, the parent directory of file operating is the data folder of the plugin
+		:param encoding: The encoding method to write the config file
 		"""
 		config_file_path = os.path.join(self.get_data_folder(), file_name) if in_data_folder else file_name
 		if isinstance(config, Serializable):
 			data = config.serialize()
 		else:
 			data = config
-		with open(config_file_path, 'w') as file:
+		target_folder = os.path.dirname(config_file_path)
+		if len(target_folder) > 0 and not os.path.isdir(target_folder):
+			os.makedirs(target_folder)
+		with open(config_file_path, 'w', encoding=encoding) as file:
+			# config file should be nicely readable, so here come the indent and non-ascii chars
 			json.dump(data, file, indent=4, ensure_ascii=False)
