@@ -46,10 +46,6 @@ class SyncStdoutStreamHandler(logging.StreamHandler):
 		with self.__write_lock:
 			super().emit(record)
 
-	def flush(self) -> None:
-		with self.__write_lock:
-			super().flush()
-
 	@classmethod
 	def update_stdout(cls, stream=None):
 		if stream is None:
@@ -57,9 +53,11 @@ class SyncStdoutStreamHandler(logging.StreamHandler):
 		with cls.__instance_lock:
 			instances = list(cls.__instances)
 		for inst in instances:
-			with cls.__write_lock:
-				inst.flush()
+			inst.acquire()  # use Handler's lock
+			try:
 				inst.stream = stream
+			finally:
+				inst.release()
 
 
 class MCColoredFormatter(ColoredFormatter):
@@ -143,10 +141,9 @@ class MCDReforgedLogger(logging.Logger):
 
 	debug_options = {}  # type: Dict[DebugOption, bool]
 
-	def __init__(self, mcdr_server, plugin_id=None):
+	def __init__(self, plugin_id: Optional[str] = None):
 		super().__init__(self.DEFAULT_NAME)
-		self.mcdr_server = mcdr_server
-		self.file_handler = None
+		self.file_handler: Optional[logging.FileHandler] = None
 
 		self.console_handler = SyncStdoutStreamHandler()
 		self.console_handler.setFormatter(self.get_console_formatter(plugin_id))
@@ -193,6 +190,12 @@ class MCDReforgedLogger(logging.Logger):
 		self.file_handler.setFormatter(self.FILE_FMT)
 		self.addHandler(self.file_handler)
 
+	def unset_file(self):
+		if self.file_handler is not None:
+			self.removeHandler(self.file_handler)
+			self.file_handler.close()
+			self.file_handler = None
+
 
 class ServerLogger(logging.Logger):
 	server_fmt = MCColoredFormatter('[%(name)s] %(message)s')
